local _Questic = _Questic_ADDON_Globals;
local Widgets = _Avyr_GUI_Widgets_Lib
local GetGossipOptions, getn, type, UIParent, GetCursorPosition, wipe, insert, format = GetGossipOptions, getn, type, UIParent, GetCursorPosition, table.wipe, table.insert, string.format;
local CloseGossip, SelectGossipOption, ipairs = CloseGossip, SelectGossipOption, ipairs;
local GossipFrameNpcNameText, GossipFrame = GossipFrameNpcNameText, GossipFrame;

local QuickGossipFrame;
local PreviewFrame;

local GossipOptions = {};
local GossipBtnTable = {};

_Questic.GossipQuickSelect = {};

local function FormatMoney(Money, Short)
    local gold = floor(Money / 10000)
    local silver = floor(Money / 100) - gold * 100;
    local copper = Money - silver * 100 - gold * 10000;
    local MoneyString = "";
    if gold > 0 then
        MoneyString = gold .. "|cFFFFD700g|r";
    end
    if silver > 0 or (gold > 0 and not Short) then
        MoneyString = MoneyString .. " " .. silver .. "|cFFC0C0C0s|r"
    end
    if copper > 0 or (gold > 0 and not Short) or (silver > 0 and not Short) then
        MoneyString = MoneyString .. " " .. copper .. "|cFFb87333c|r";
    end
    return MoneyString
end


local function SetupFrame()
    if not QuickGossipFrame then
        QuickGossipFrame = Widgets:CreateContainer(UIParent);
        local Midx, Midy = UIParent:GetCenter();
        local Mousex, Mousey = GetCursorPosition();
        Mousey = Mousey / UIParent:GetEffectiveScale();
        Mousex = Mousex / UIParent:GetEffectiveScale();
        local Point = "";
        if Midy < Mousey then
            Point = "TOP"
        else
            Point = "BOTTOM"
        end
        if Midx < Mousex then
            Point = Point .. "RIGHT"
        else
            Point = Point .. "LEFT"
        end
        local NewScale = _Questic.GetVariable("GossipQuickSelectScale");
        QuickGossipFrame:SetScale(NewScale);
        QuickGossipFrame:SetPoint(Point, UIParent, "BOTTOMLEFT", Mousex / NewScale, Mousey / NewScale);
    else
        local Point, _, _, x, y = QuickGossipFrame:GetPoint(1);
        if not x or not y then
            x, y = GetCursorPosition();
            x = x / UIParent:GetEffectiveScale();
            y = y / UIParent:GetEffectiveScale();
        else
            local PrevScale = QuickGossipFrame:GetScale();
            x = x * PrevScale;
            y = y * PrevScale;
        end
        local NewScale = _Questic.GetVariable("GossipQuickSelectScale");
        QuickGossipFrame:SetScale(NewScale);
        QuickGossipFrame:SetPoint(Point, UIParent, "BOTTOMLEFT", x / NewScale, y / NewScale);
    end
    QuickGossipFrame:PresetBorders()
    QuickGossipFrame:SetFrameStrata("HIGH")
    QuickGossipFrame:EnableMouse(true)
    QuickGossipFrame:Size(250, 200);
end

_Questic.GossipQuickSelect.Update = function(self)
    if PreviewFrame then
        _Questic.GossipQuickSelect:ShowPreviewFrame(true);
    end
    if not _Questic.IsEnabled() or not _Questic.GetVariable("GossipQuickSelect") then
        if QuickGossipFrame then
            QuickGossipFrame:Release();
            QuickGossipFrame = nil;
        end
        return nil;
    end

    if (GossipFrame and GossipFrame:IsShown()) or (QuestFrameGreetingPanel and QuestFrameGreetingPanel:IsShown() and _Questic.GetVariable("IncludeQuestsInGosseipSelect")) then
        SetupFrame();
        GossipOptions = { GetGossipOptions() };
        GossipOptions = GossipOptions or {};
        wipe(GossipBtnTable);
        if _Questic.GetVariable("IncludeQuestsInGosseipSelect") then
            local ActiveQuests, AvailiableQuests = _Questic.FetchOfferedQuests();
            for i, v in ipairs(ActiveQuests) do
                local r, g, b = 1, 1,1 ;
                if v.Complete then
                    r, g, b = 1, 0.82, 0;
                end
                insert(GossipBtnTable, {
                    Type = "DB_BL",
                    Icon = "Interface\\AddOns\\Questic\\Media\\ActiveQuestIconMask.blp",
                    IconData = { 0, 1, 0, 1, r, g, b },
                    Text = v.Name,
                    Tooltip = v.Name,
                    Alignment = "LEFT",
                    Func = function()
                        _Questic.SelectQuest(v.Index, "Active", v.Type)
                    end
                })
            end
            for i, v in ipairs(AvailiableQuests) do
                local r, g, b = 1, 0.82, 0;
                if v.Legendary then
                    r, g, b = 1, 0.41, 0;
                elseif v.Frequency ~= 1 then
                    r, g, b = 0, 0.82, 1;
                end
                local QuickQuestEnabled = _Questic.IsQuickQuest(v.QuestID)
                if QuickQuestEnabled and _Questic.GetVariable("QuickQuestEnable") then
                    insert(GossipBtnTable, {
                        Type = "DB_BL",
                        Icon = "Interface\\AddOns\\Questic\\Media\\AvQuestIconMask.tga",
                        IconData = { 0, 1, 0, 1, r, g, b },
                        Text = v.Name,
                        Tooltip = v.Name .. "\n\nQuickQuest: Right-click to automically " .. QuickQuestEnabled,
                        Alignment = "LEFT",
                        Func = function(_, Button)
                            if Button == "RightButton" then
                                _Questic.SetActiveQuickQuest(v.Name)
                            end
                            _Questic.SelectQuest(v.Index, "Available", v.Type) --add quick select function
                        end
                    })
                else
                    insert(GossipBtnTable, {
                        Type = "DB_BL",
                        Icon = "Interface\\AddOns\\Questic\\Media\\AvQuestIconMask.blp",
                        IconData = { 0, 1, 0, 1, r, g, b },
                        Text = v.Name,
                        Tooltip = v.Name,
                        Alignment = "LEFT",
                        Func = function()
                            _Questic.SelectQuest(v.Index, "Available", v.Type)
                        end
                    })
                end
            end
        end
        for i = 1, getn(GossipOptions) / 2 do
            insert(GossipBtnTable, {
                Type = "DB_BL",
                Icon = "Interface\\GossipFrame\\" .. GossipOptions[2 + (i - 1) * 2] .. "GossipIcon.blp",
                Text = GossipOptions[1 + (i - 1) * 2],
                Tooltip = GossipOptions[1 + (i - 1) * 2],
                Alignment = "LEFT",
                Func = function(_, Button)
                    if not _Questic.GetVariable("RightClickGossipAdd") or Button == "LeftButton" then
                        if GossipOptions[2 + (i - 1) * 2]:lower() == "gossip" or GossipOptions[2 + (i - 1) * 2]:lower() == "chatbubble" then
                            _Questic.GossipFunctions.FetchLastGossip(GossipOptions[1 + (i - 1) * 2], GossipFrameNpcNameText:GetText());
                        end
                        SelectGossipOption(i);
                    elseif _Questic.GetVariable("RightClickGossipAdd") and Button == "RightButton" and GossipFrameNpcNameText:GetText() and GossipOptions[1 + (i - 1) * 2] then
                        _Questic.GossipFunctions.ChangeGossip("UserDefined", nil, "ADD", { NPC = GossipFrameNpcNameText:GetText(), Gossip = GossipOptions[1 + (i - 1) * 2] });
                    end
                end
            })
        end
        insert(GossipBtnTable, {
            Type = "DB_BL",
            Text = "Close",
            Alignment = "LEFT",
            Icon = "Interface\\AddOns\\Questic\\Media\\PlusMask.blp",
            IconData = { 0, 1, 0, 1, 1, 0.25, 0.25, false, 45 },
            VSpacing = getn(GossipBtnTable) == 0 and 0 or 10,
            Func = function()
                CloseGossip()
            end
        })
    elseif QuestFrameProgressPanel and QuestFrameProgressPanel:IsShown() and _Questic.GetVariable("IncludeQuestsInGosseipSelect") then
        SetupFrame();
        wipe(GossipBtnTable);
        insert(GossipBtnTable, {
            Type = "LB",
            Text = GetTitleText(),
            Alignment = "LEFT",
        })
        if GetNumQuestItems() > 0 then --items
            for i = 1, GetNumQuestItems() do
                if GetQuestItemInfo("required", i) ~= "" then
                    local Name, Texture, Amount = GetQuestItemInfo("required", i);
                    insert(GossipBtnTable, {
                        Type = "DB_BL",
                        Icon = Texture,
                        IconData = { 0.1, 0.9, 0.1, 0.9 },
                        Text = Amount .. "x " .. Name,
                        Tooltip = "SetQuestItem",
                        SpecialTooltip = { "required", i },
                        Alignment = "LEFT",
                    })
                end
            end
        end
        if GetNumQuestCurrencies() > 0 then --currency
            for i = 1, GetNumQuestCurrencies() do
                if GetQuestCurrencyInfo("required", i) ~= "" then
                    local Name, Texture, Amount = GetQuestCurrencyInfo("required", i);
                    insert(GossipBtnTable, {
                        Type = "DB_BL",
                        Icon = Texture,
                        IconData = { 0.1, 0.9, 0.1, 0.9 },
                        Text = Amount .. "x " .. Name,
                        Tooltip = "SetQuestCurrency",
                        SpecialTooltip = { "required", i },
                        Alignment = "LEFT",
                    })
                end
            end
        end
        if GetQuestMoneyToGet() > 0 then
            insert(GossipBtnTable, {
                Type = "DB_BL",
                Icon = 133784,
                IconData = { 0.1, 0.9, 0.1, 0.9 },
                Text = FormatMoney(GetQuestMoneyToGet()),
                Tooltip = FormatMoney(GetQuestMoneyToGet()),
                Alignment = "LEFT",
            })
        end
        insert(GossipBtnTable, {
            Type = "DB_BL",
            Text = "Confirm",
            Icon = "Interface\\AddOns\\Questic\\Media\\CheckMask.blp",
            IconData = { 0, 1, 0, 1, 0.25, 1, 0.25, false, 135 },
            Alignment = "LEFT",
            Disable = not IsQuestCompletable(),
            Func = function()
                CompleteQuest()
            end
        })
        insert(GossipBtnTable, {
            Type = "DB_BL",
            Text = "Cancel",
            Icon = "Interface\\AddOns\\Questic\\Media\\PlusMask.blp",
            IconData = { 0, 1, 0, 1, 1, 0.25, 0.25, false, 45 },
            Alignment = "LEFT",
            Func = function()
                CloseQuest()
            end
        })
    elseif QuestFrameRewardPanel and QuestFrameRewardPanel:IsShown() and _Questic.GetVariable("IncludeQuestsInGosseipSelect") then --reward/turn in frame
        SetupFrame();
        wipe(GossipBtnTable);
        insert(GossipBtnTable, {
            Type = "LB",
            Text = GetTitleText(),
            Alignment = "LEFT",
        })
        for i = 1, GetNumQuestChoices() do
            if GetQuestItemInfo("choice", i) ~= "" then
                if getn(GossipBtnTable) == 1 then
                    insert(GossipBtnTable, {
                        Type = "LB",
                        Text = "Select a reward to complete the quest",
                        Alignment = "LEFT",
                    })
                end
                local Name, Texture, QuestCount = GetQuestItemInfo("choice", i);
                insert(GossipBtnTable, {
                    Type = "DB_BL",
                    Icon = Texture,
                    IconData = { 0.1, 0.9, 0.1, 0.9 },
                    Text = Name,
                    Tooltip = "SetQuestItem",
                    SpecialTooltip = { "choice", i },
                    Alignment = "LEFT",
                    Func = function()
                        GetQuestReward(i);
                    end
                })
            end
        end
        local NumRewardsForSelection = getn(GossipBtnTable);
        for i = 1, GetNumQuestRewards() do
            if GetQuestItemInfo("reward", i) ~= "" then
                if getn(GossipBtnTable) == NumRewardsForSelection then
                    insert(GossipBtnTable, {
                        Type = "LB",
                        Text = "Rewards",
                        Alignment = "LEFT",
                    })
                end
                local Name, Texture, QuestCount = GetQuestItemInfo("reward", i);
                insert(GossipBtnTable, {
                    Type = "DB_BL",
                    Icon = Texture,
                    IconData = { 0.1, 0.9, 0.1, 0.9 },
                    Text = Name,
                    Tooltip = "SetQuestItem",
                    SpecialTooltip = { "reward", i },
                    Alignment = "LEFT",
                })
            end
        end
        for i = 1, GetNumRewardCurrencies() do
            if GetQuestCurrencyInfo("reward", i) ~= "" then
                if getn(GossipBtnTable) == NumRewardsForSelection then
                    insert(GossipBtnTable, {
                        Type = "LB",
                        Text = "Rewards",
                        Alignment = "LEFT",
                    })
                end
                local Name, Texture, QuestCount = GetQuestCurrencyInfo("reward", i);
                insert(GossipBtnTable, {
                    Type = "DB_BL",
                    Icon = Texture,
                    IconData = { 0.1, 0.9, 0.1, 0.9 },
                    Text = Name,
                    Tooltip = "SetQuestCurrency",
                    SpecialTooltip = { "reward", i },
                    Alignment = "LEFT",
                })
            end
        end
        if NumRewardsForSelection < 2 then
            insert(GossipBtnTable, {
                Type = "DB_BL",
                Text = "Complete quest",
                Icon = "Interface\\AddOns\\Questic\\Media\\CheckMask.blp",
                IconData = { 0, 1, 0, 1, 0.25, 1, 0.25, false, 135 },
                Alignment = "LEFT",
                Func = function()
                    GetQuestReward(1);
                end
            })
        end
        insert(GossipBtnTable, {
            Type = "DB_BL",
            Text = "Cancel",
            Icon = "Interface\\AddOns\\Questic\\Media\\PlusMask.blp",
            IconData = { 0, 1, 0, 1, 1, 0.25, 0.25, false, 45 },
            Alignment = "LEFT",
            Func = function()
                CloseQuest()
            end
        })
    elseif QuestFrameDetailPanel and QuestFrameDetailPanel:IsShown() and _Questic.GetVariable("IncludeQuestsInGosseipSelect") then
        SetupFrame();
        wipe(GossipBtnTable);
        insert(GossipBtnTable, {
            Type = "LB",
            Text = GetTitleText(),
            Alignment = "LEFT",
        })
        insert(GossipBtnTable, {
            Type = "DB_BL",
            Text = "Accept quest",
            Icon = "Interface\\AddOns\\Questic\\Media\\CheckMask.blp",
            IconData = { 0, 1, 0, 1, 0.25, 1, 0.25, false, 135 },
            Alignment = "LEFT",
            Func = function()
                QuestDetailAcceptButton_OnClick();
            end
        })
        insert(GossipBtnTable, {
            Type = "DB_BL",
            Text = "Cancel",
            Icon = "Interface\\AddOns\\Questic\\Media\\PlusMask.blp",
            IconData = { 0, 1, 0, 1, 1, 0.25, 0.25, false, 45 },
            Alignment = "LEFT",
            Func = function()
                CloseQuest()
            end
        })
    else
        if QuickGossipFrame then
            QuickGossipFrame:Release();
            QuickGossipFrame = nil;
        end
        return nil;
    end
    local Height = QuickGossipFrame:AddItems(GossipBtnTable);
    QuickGossipFrame:Size(nil, Height);
end

_Questic.GossipQuickSelect.ShowPreviewFrame = function(self, Show)
    if type(Show) == "nil" then
        Show = type(PreviewFrame) == "nil";
    end
    if not Show then
        if PreviewFrame then
            PreviewFrame:Release();
            PreviewFrame = nil;
        end
        return nil;
    end
    if not PreviewFrame then
        PreviewFrame = Widgets:CreateContainer(UIParent);
        PreviewFrame:SetText("Gossip Sample frame, Move me!", nil, nil, nil, nil, nil, "TOP")
        PreviewFrame:MakeMovable(true);
        local Midx, Midy = UIParent:GetCenter();
        local Mousex, Mousey = GetCursorPosition();
        Mousey = Mousey / UIParent:GetEffectiveScale();
        Mousex = Mousex / UIParent:GetEffectiveScale();
        local Point = "";
        if Midy < Mousey then
            Point = "TOP"
        else
            Point = "BOTTOM"
        end
        if Midx < Mousex then
            Point = Point .. "RIGHT"
        else
            Point = Point .. "LEFT"
        end
        local NewScale = _Questic.GetVariable("GossipQuickSelectScale");
        PreviewFrame:Size(250, 200);
        PreviewFrame:SetScale(NewScale);
        PreviewFrame:SetPoint(Point, UIParent, "BOTTOMLEFT", Mousex / NewScale, Mousey / NewScale);

    else
        local Point, _, _, x, y = PreviewFrame:GetPoint(1);
        if not x or not y then
            x, y = GetCursorPosition();
            x = x / UIParent:GetEffectiveScale();
            y = y / UIParent:GetEffectiveScale();
        else
            local PrevScale = PreviewFrame:GetScale();
            x = x * PrevScale;
            y = y * PrevScale;
        end
        local NewScale = _Questic.GetVariable("GossipQuickSelectScale");
        PreviewFrame:SetScale(NewScale);
        PreviewFrame:SetPoint(Point, UIParent, "BOTTOMLEFT", x / NewScale, y / NewScale);
    end
    wipe(GossipBtnTable);
    local Space = 10;
    if _Questic.GetVariable("IncludeQuestsInGosseipSelect") then
        GossipBtnTable = {
            {
                VSpacing = Space,
                Type = "DB_BL",
                Text = format("|TInterface\\GossipFrame\\AvailableQuestIcon.blp:0|t") .. " Sample quest item",
                Tooltip = "Sample quest item",
                Alignment = "LEFT",
                Func = function() _Questic.GossipQuickSelect:ShowPreviewFrame(false) end,
            }
        }
        Space = 0;
    end
    insert(GossipBtnTable, {
        VSpacing = Space,
        Type = "DB_BL",
        Text = format("|TInterface\\GossipFrame\\GossipGossipIcon.blp:0|t") .. " Sample Gossip item",
        Tooltip = "Sample quest item",
        Alignment = "LEFT",
        Func = function() _Questic.GossipQuickSelect:ShowPreviewFrame(false) end,
    })
    insert(GossipBtnTable, {
        Type = "DB_BL",
        Text = format("|TInterface\\GossipFrame\\VendorGossipIcon.blp:0|t") .. " Sample Vendor",
        Tooltip = "Sample quest item",
        Alignment = "LEFT",
        Func = function() _Questic.GossipQuickSelect:ShowPreviewFrame(false) end,
    })
    insert(GossipBtnTable, {
        Type = "DB_BL",
        Text = "Close",
        VSpacing = 10,
        Func = function() _Questic.GossipQuickSelect:ShowPreviewFrame(false) end,
    })
    local Height = PreviewFrame:AddItems(GossipBtnTable);
    PreviewFrame:Size(nil, Height);
end


