local _Questic = _Questic_ADDON_Globals;

local function DisableBlizzQuestFrame(Show)
    QuestFrameGoodbyeButton:EnableMouse(Show);
    QuestFrameCompleteQuestButton:EnableMouse(Show);
    QuestFrameCloseButton:EnableMouse(Show);
    QuestFrameAcceptButton:EnableMouse(Show);
    QuestFrameDeclineButton:EnableMouse(Show);
    QuestFrame:EnableMouse(Show)
    QuestFrame:SetAlpha(Show and 1 or 0);
end

_Questic.QuestFrameSuppress = function(Show)
    if _Questic.IsEnabled() and _Questic.GetVariable("GossipQuickSelect") and _Questic.GetVariable("IncludeQuestsInGosseipSelect") and _Questic.GetVariable("HideBlizzQuestFrame") then
        DisableBlizzQuestFrame(false)
    else
        DisableBlizzQuestFrame(true)
    end
end

_Questic.SelectQuest = function(Index, QuestType, NPCType)
    if QuestType == "Available" then
        SelectAvailableQuest(Index);
        return SelectGossipAvailableQuest(Index);
    else
        SelectGossipActiveQuest(Index);
        return SelectActiveQuest(Index);
    end
end

_Questic.FetchOfferedQuests = function()
    local ActiveQuestTable = {};
    local AvailableQuestTable = {};
    local ActiveGossipQuests = { GetGossipActiveQuests() };
    --title, level, isLowLevel, isComplete, isLegendary, isIgnored
    for i = 1, GetNumGossipActiveQuests() do
        local title = ActiveGossipQuests[1 + 7 * (i - 1)];
        local isComplete = ActiveGossipQuests[4 + 7 * (i - 1)];
        local QuestID = ActiveGossipQuests[7 + 7 * (i - 1)];
        table.insert(ActiveQuestTable, { Index = i, Name = title, Complete = isComplete, Type = "Gossip", QuestID = QuestID });
    end
    local AvailableGossipQuests = { GetGossipAvailableQuests() };
    --title, level, isTrivial, frequency, isRepeatable, isLegendary, isIgnored
    for i = 1, GetNumGossipAvailableQuests() do

        local title = AvailableGossipQuests[1 + 8 * (i - 1)];
        local isTrivial = AvailableGossipQuests[3 + 8 * (i - 1)];
        local frequency = AvailableGossipQuests[4 + 8 * (i - 1)];
        local isLegendary = AvailableGossipQuests[6 + 8 * (i - 1)];
        local QuestID = AvailableGossipQuests[8 + 8 * (i - 1)];
        table.insert(AvailableQuestTable, { Index = i, Name = title, Frequency = frequency, Trivial = isTrivial, Legendary = isLegendary, Type = "Gossip", QuestID = QuestID });
    end
    for i = 1, GetNumActiveQuests() do
        local title, isComplete = GetActiveTitle(i);
        table.insert(ActiveQuestTable, { Index = i, Name = title, Complete = isComplete, Type = "Greeting" });
    end
    for i = 1, GetNumAvailableQuests() do
        local isTrivial, frequency, isRepeatable, isLegendary = GetAvailableQuestInfo(i);
        local title = GetAvailableTitle(i);
        table.insert(AvailableQuestTable, { Index = i, Name = title, Frequency = frequency, Trivial = isTrivial, Legendary = isLegendary, Type = "Greeting" });
    end
    return ActiveQuestTable, AvailableQuestTable;
end

local ToIgnoreTags = {
    128, --Emissary
    140, --Weekly rated award
};

local function QuestDisplayChecker(QuestIndex)
    if not GetQuestLogTitle(QuestIndex) then --nil
        return false;
    end
    local QuestTag, _, WQ = GetQuestTagInfo(select(8, GetQuestLogTitle(QuestIndex)));
    if type(QuestTag) == "nil" then
        return true;
    elseif WQ then
        return false;
    else
        for i, v in pairs(ToIgnoreTags) do
            if v == QuestTag then
                return false;
            end
        end
        return true;
    end
end

_Questic.ShareQuest = function(QuestIndex, QuestID)
    if QuestID and not QuestIndex then
        QuestIndex = GetQuestLogIndexByID(QuestID);
    end
    if QuestDisplayChecker(QuestIndex) then
        SelectQuestLogEntry(QuestIndex)
        if GetQuestLogPushable() then
            QuestLogPushQuest();
        end
    end
end


_Questic.GetQuestLogTable = function()
    local QuestTable = {};
    local LastHeaderIndex = 0;
    local HeaderID = "Misc";
    for i = 1, GetNumQuestLogEntries() do
        local Name, level, isGroupQ, isHeader, _, _, _, questID, _, _, IsOnMap = GetQuestLogTitle(i);
        local QuestTag, _, WQ = GetQuestTagInfo(select(8, GetQuestLogTitle(i)));
        local Pushable = "0";
        SelectQuestLogEntry(i);
        if GetQuestLogPushable() then
            Pushable = "1";
        end
        if isHeader then
            QuestTable[Name] = { Name = Name, Type = "H", QuestIndex = i, Pushable = Pushable, HeaderID = "", OnMap = IsOnMap };
            HeaderID = Name;
        elseif QuestDisplayChecker(i) then
            QuestTable[questID] = { Name = Name, Type = "NQ", QuestIndex = i, Pushable = Pushable, HeaderID = HeaderID, OnMap = IsOnMap };
        else
            QuestTable[questID] = { Name = Name, Type = "Q", QuestIndex = i, Pushable = Pushable, HeaderID = HeaderID, OnMap = IsOnMap };
        end
    end
    return QuestTable;
end

local UsedList = {};
_Questic.QuestItemList = {};
local RemoveIndex = {};
_Questic.BagScan = function(BagID)
    table.wipe(_Questic.QuestItemList);
    for BagID = 0, 4 do
        for SlotID = 1, GetContainerNumSlots(BagID) do
            local QuestItem, QuestID, Accepted = GetContainerItemQuestInfo(BagID, SlotID);
            if QuestItem and GetItemSpell(select(7, GetContainerItemInfo(BagID, SlotID))) then
                table.insert(_Questic.QuestItemList, { SlotID = SlotID, BagID = BagID });
            elseif QuestID and not UsedList[QuestID] and _Questic.GetVariable("Enable") and _Questic.GetVariable("AutoAccept") and _Questic.GetVariable("AutoAcceptQuestsInBag") and not Accepted and not IsQuestFlaggedCompleted(QuestID) then
                if _Questic.GetVariable("AutoAcceptQuestsInBag") and _Questic.GetVariable("AutoAccept") then
                    UseContainerItem(BagID, SlotID);
                    UsedList[QuestID] = true;
                end
            end
        end
    end
end

_Questic.QuestItemBlackList = function(Action, ItemName)
    if Action == "Check" then
        if _Questic.GetVariable("QuestItemBarEnableBlackList") then
            if type(ItemName) == "string" then
                local BlackList = _Questic.GetVariable("QuestItemBlackList");
                if BlackList[ItemName] then
                    return true;
                else
                    return false;
                end
            else
                return true;
            end
        else
            return false;
        end
    elseif Action == "Add" and type(ItemName) == "string" and ItemName ~= "" and GetItemInfo(ItemName) then
        local BlackList = _Questic.GetVariable("QuestItemBlackList");
        if BlackList[ItemName] then
            _Questic.RefreshAddonFrames("QuestItemBlackList")
            print("Questic: \"" .. ItemName .. "\" already in quest item bar blacklist.")
        else
            BlackList[ItemName] = true;
            _Questic.RefreshAddonFrames("QuestItemBlackList")
            print("Questic: \"" .. ItemName .. "\" added to quest item bar blacklist.")
        end
    elseif Action == "Remove" and type(ItemName) == "string" and ItemName ~= "" then
        local BlackList = _Questic.GetVariable("QuestItemBlackList");
        BlackList[ItemName] = nil;
        _Questic.RefreshAddonFrames("QuestItemBlackList")
        print("Questic: \"" .. ItemName .. "\" removed from quest item bar blacklist.")
    end
end

local function GetQuestNameFromID(QuestID)
    GameTooltip:SetOwner(UIParent)
    GameTooltip:SetHyperlink("\124cffffff00\124Hquest:" .. QuestID .. ":-1\124h[DUMMY QUEST]\124h\124r");
    local Name = GameTooltipTextLeft1:GetText() or ""
    GameTooltip:Hide();
    return Name;
end



local QuickQuestList = {
    --[43895] = "Seal for gold",
    --[43892] = function() return "Seal for resources" end,
    --[47851] = function() return "Seal for marks" end,

    --BfA
    [52835] = function() return "exchange 10 [" .. (GetItemInfo("\124cff0070dd\124Hitem:137642::::::::120:::::\124h[Mark of Honor]\124h\124r") or "Loading") .. "] for a [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:1580:0|h[Valor Points]|h|r") or "Loading") .. "]" end, --Seal of Wartorn Fate: Marks of Honor
    [52839] = function() return "exchange 25 [" .. (GetItemInfo("\124cff0070dd\124Hitem:137642::::::::120:::::\124h[Mark of Honor]\124h\124r") or "Loading") .. "] for a [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:1580:0|h[Valor Points]|h|r") or "Loading") .. "]" end, --Seal of Wartorn Fate: Additional Marks of Honor
    [52834] = function() return "exchange 2,000 gold for a [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:1580:0|h[Valor Points]|h|r") or "Loading") .. "]" end, --Seal of Wartorn Fate: Gold
    [52838] = function() return "exchange 5,000 gold for a [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:1580:0|h[Valor Points]|h|r") or "Loading") .. "]" end, --Seal of Wartorn Fate: Piles of Gold
    [52837] = function() return "exchange 250 [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:824:0|h[Valor Points]|h|r") or "Loading") .. "] for a [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:1580:0|h[Valor Points]|h|r") or "Loading") .. "]" end, --Seal of Wartorn Fate: War Resources
    [52840] = function() return "exchange 500 [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:824:0|h[Valor Points]|h|r") or "Loading") .. "] for a [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:1580:0|h[Valor Points]|h|r") or "Loading") .. "]" end, --Seal of Wartorn Fate: Stashed War Resources
}

local CompiledQuickQuestList = {};
local ActiveQuest;

_Questic.CompileQuickQuestList = function()
    table.wipe(CompiledQuickQuestList);
    for i, v in pairs(QuickQuestList) do
        if C_QuestLog.GetQuestInfo(i) then
        CompiledQuickQuestList[C_QuestLog.GetQuestInfo(i)] = v
            end
        end
end

_Questic.IsQuickQuest = function(ID)
    if type(ID) == "number" then
        if QuickQuestList[ID] then
            return QuickQuestList[ID]();
        else
            return nil
        end
    elseif type(ID) == "string" then
        if CompiledQuickQuestList[ID] then
            return CompiledQuickQuestList[ID]()
        else
            return nil
        end
    end
end

_Questic.ClearActiveQuickQuest = function()
    ActiveQuest = nil;
end

_Questic.SetActiveQuickQuest = function(Quest)
    ActiveQuest = Quest;
end

_Questic.GetActiveQuickQuest = function()
    return ActiveQuest or ""
end

local QuestsToBeTracked = {}
_Questic.UpdateQuestLog = function()
    if _Questic.GetVariable("Enable") and _Questic.GetVariable("AutoTrack") then
        if  (_Questic.GetVariable("UntrackDungeon") and select(2,GetInstanceInfo()) == "party") 
        or (_Questic.GetVariable("UntrackRaid") and select(2,GetInstanceInfo()) == "raid") then 
            local Questlog = _Questic.GetQuestLogTable()
            for i,v in pairs(Questlog) do 
                RemoveQuestWatch(v.QuestIndex)
            end             
        else 
            wipe(QuestsToBeTracked)
            local MapID = C_Map.GetBestMapForUnit("player")
            --[[local ParentMapID = C_Map.GetMapInfo(MapID).parentMapID
            local minX, maxX, minY, maxY = 0,1,0,1
            if type(ParentMapID) == "number" then 
                minX, maxX, minY, maxY = C_Map.GetMapRectOnMap(MapID, ParentMapID)
            end ]]
            if type(MapID) == "number" then 
                local Quests = C_QuestLog.GetQuestsOnMap(MapID)
                if type(Quests) == "table" then 
                    for i,v in ipairs(Quests) do 
                        QuestsToBeTracked[v.questID] = {x = v.x, y = v.y} 
                    end 
                end 
            end 
            local Questlog = _Questic.GetQuestLogTable()
            for i,v in pairs(Questlog) do 
                --if QuestsToBeTracked[i] then 
                --print(QuestsToBeTracked[i].x,QuestsToBeTracked[i].y)
                --print(minX, maxX, minY, maxY)
                --end 
                if QuestsToBeTracked[i] 
                --and QuestsToBeTracked[i].x >= minX and QuestsToBeTracked[i].x <= maxX and QuestsToBeTracked[i].y >= minY and QuestsToBeTracked[i].y <= maxY 
                then 
                    AddQuestWatch(v.QuestIndex)
                else 
                    RemoveQuestWatch(v.QuestIndex)
                end 
            end 
        end 
    end 
end 
